from sqlalchemy import and_

from database import db, ITEMS_PER_PAGE
from flask_covid19.blueprints.app_all.all_model import BlueprintDateReported, BlueprintLocationGroup, BlueprintLocation


class RkiMeldedatum(BlueprintDateReported):
    __tablename__ = 'rki_date_reported'
    __mapper_args__ = {'concrete': True}
    __table_args__ = (
        db.UniqueConstraint(
            'date_reported_import_str',
            'datum',
            'year_day_of_year',
            name="uix_rki_cases_date_reported"
        ),
    )

    def __str__(self):
        return " " + self.datum.isoformat() + " "

    id = db.Column(db.Integer, primary_key=True)
    datum = db.Column(db.Date, nullable=False, unique=True)
    year = db.Column(db.Integer, nullable=False)
    month = db.Column(db.Integer, nullable=False)
    day_of_month = db.Column(db.Integer, nullable=False)
    day_of_week = db.Column(db.Integer, nullable=False)
    week_of_year = db.Column(db.Integer, nullable=False)
    day_of_year = db.Column(db.Integer, nullable=True)
    year_week = db.Column(db.String(255), nullable=False)
    year_day_of_year = db.Column(db.String(255), nullable=False, unique=True)
    date_reported_import_str = db.Column(db.String(255), nullable=False, unique=True)
    processed_update = db.Column(db.Boolean, nullable=False)
    processed_full_update = db.Column(db.Boolean, nullable=False)

    @classmethod
    def create_new_object_factory(cls, my_date_rep: str):
        o = cls.create_new_object_factory_for_rki(my_date_rep)
        oo = RkiMeldedatum(
            datum=o.datum,
            year=o.year,
            month=o.month,
            day_of_month=o.day_of_month,
            day_of_week=o.day_of_week,
            week_of_year=o.week_of_year,
            day_of_year=o.day_of_year,
            year_week=o.year_week,
            year_day_of_year=o.year_day_of_year,
            date_reported_import_str=o.date_reported_import_str,
            processed_update=o.processed_update,
            processed_full_update=o.processed_full_update,
        )
        return oo


class RkiBundesland(BlueprintLocationGroup):
    __tablename__ = 'rki_landkreis_bundesland'
    __mapper_args__ = {'concrete': True}
    __table_args__ = (
        db.UniqueConstraint('id_bundesland', 'location_group', name="uix_rki_landkreis_bundesland"),
    )

    def __str__(self):
        return " " + self.location_group + " "

    id = db.Column(db.Integer, primary_key=True)
    id_bundesland = db.Column(db.String(255), nullable=False)
    location_group = db.Column(db.String(255), nullable=False)
    processed_update = db.Column(db.Boolean, nullable=False)
    processed_full_update = db.Column(db.Boolean, nullable=False)


class RkiLandkreis(BlueprintLocation):
    __tablename__ = 'rki_landkreis'
    __mapper_args__ = {'concrete': True}
    __table_args__ = (
        db.UniqueConstraint('location', 'id_landkreis', name="uix_rki_landkreis"),
    )

    def __str__(self):
        return " " + str(self.location_group) + " : " + self.location + " ( " + self.id_landkreis + " ) "

    id = db.Column(db.Integer, primary_key=True)
    id_landkreis = db.Column(db.String(255), nullable=False)
    location = db.Column(db.String(255), nullable=False)
    location_group_id = db.Column(db.Integer, db.ForeignKey('rki_landkreis_bundesland.id'), nullable=False)
    location_group = db.relationship(
        'RkiBundesland',
        lazy='joined',
        cascade='save-update',
        order_by='RkiBundesland.location_group')
    processed_update = db.Column(db.Boolean, nullable=False)
    processed_full_update = db.Column(db.Boolean, nullable=False)

    @classmethod
    def remove_all(cls):
        num_rows_deleted = 0
        try:
            num_rows_deleted = db.session.query(cls).delete()
            db.session.commit()
        except Exception:
            db.session.rollback()
        return num_rows_deleted

    @classmethod
    def get_all_as_page(cls, page):
        return db.session.query(cls) \
            .order_by(cls.location.asc()) \
            .paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_all(cls):
        return db.session.query(cls)\
            .order_by(cls.location.asc())\
            .all()

    @classmethod
    def get_all_as_dict(cls):
        locations = {}
        for my_location in cls.get_all():
            locations[my_location.location] = my_location
        return locations

    @classmethod
    def get_by_id(cls, other_id):
        return db.session.query(cls)\
            .filter(cls.id == other_id)\
            .one()

    @classmethod
    def get_bochum(cls):
        return db.session.query(cls)\
            .filter(cls.location == 'SK Bochum')\
            .one()

    @classmethod
    def find_by_country_code_and_country_and_who_region_id(cls, my_id_landkreis, i_location_group, my_location_group):
        return db.session.query(cls).filter(
            and_(
                cls.id_landkreis == my_id_landkreis,
                cls.location_group == i_location_group,
                cls.location_group_id == my_location_group.id
            )
        ).one_or_none()

    @classmethod
    def find_by_id_landkreis(cls, my_id_landkreis):
        return db.session.query(cls).filter(
            cls.id_landkreis == my_id_landkreis
        ).one_or_none()

    @classmethod
    def get_by_id_landkreis(cls, my_id_landkreis):
        return db.session.query(cls).filter(
            cls.id_landkreis == my_id_landkreis
        ).one()

    @classmethod
    def find_by_landkreis(cls, i_landkreis):
        return db.session.query(cls).filter(
            cls.landkreis == i_landkreis
        ).one_or_none()

    @classmethod
    def get_by_landkreis(cls, i_landkreis):
        return db.session.query(cls).filter(
            cls.landkreis == i_landkreis
        ).one()

    @classmethod
    def get_landkreise_for_bundesland(cls, bundesland, page):
        return db.session.query(cls).filter(
            cls.location_group == bundesland
        ).order_by(cls.country).paginate(page, per_page=ITEMS_PER_PAGE)


class RkiData(db.Model):
    __tablename__ = 'rki'

    id = db.Column(db.Integer, primary_key=True)
    fid = db.Column(db.String(255), nullable=False)
    altersgruppe = db.Column(db.String(255), nullable=False)
    geschlecht = db.Column(db.String(255), nullable=False)
    anzahl_fall = db.Column(db.String(255), nullable=False)
    anzahl_todesfall = db.Column(db.String(255), nullable=False)
    # meldedatum = db.Column(db.String(255), nullable=False)
    datenstand = db.Column(db.String(255), nullable=False)
    neuer_fall = db.Column(db.String(255), nullable=False)
    neuer_todesfall = db.Column(db.String(255), nullable=False)
    ref_datum = db.Column(db.String(255), nullable=False)
    neu_genesen = db.Column(db.String(255), nullable=False)
    anzahl_genesen = db.Column(db.String(255), nullable=False)
    ist_erkrankungsbeginn = db.Column(db.String(255), nullable=False)
    altersgruppe2 = db.Column(db.String(255), nullable=False)
    meldedatum_id = db.Column(db.Integer, db.ForeignKey('rki_date_reported.id'), nullable=False)
    meldedatum = db.relationship(
        'RkiMeldedatum',
        lazy='joined',
        cascade='save-update',
        order_by='desc(RkiMeldedatum.datum)')
    landkreis_id = db.Column(db.Integer, db.ForeignKey('rki_landkreis.id'), nullable=False)
    landkreis = db.relationship(
        'RkiLandkreis',
        lazy='joined',
        cascade='save-update',
        order_by='asc(RkiLandkreis.location)')
    processed_update = db.Column(db.Boolean, nullable=False)
    processed_full_update = db.Column(db.Boolean, nullable=False)

    @classmethod
    def remove_all(cls):
        for one in cls.get_all():
            db.session.delete(one)
        db.session.commit()
        return None

    @classmethod
    def get_all_as_page(cls, page):
        return db.session.query(cls).paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_all(cls):
        return db.session.query(cls).all()

    @classmethod
    def get_by_id(cls, other_id):
        return db.session.query(cls).filter(cls.id == other_id).one()

    @classmethod
    def find_by_id(cls, other_id):
        return db.session.query(cls).filter(cls.id == other_id).one_or_none()
