from flask import render_template, redirect, url_for, flash, Blueprint
from celery import states
from celery.utils.log import get_task_logger
from flask_admin.contrib.sqla import ModelView

from database import admin, db, celery
from flask_covid19.blueprints.app_web.web_services import vaccination_service

from flask_covid19.blueprints.data_vaccination.vaccination_model import VaccinationData
from flask_covid19.blueprints.data_vaccination.vaccination_model import VaccinationDateReported
from flask_covid19.blueprints.data_vaccination.vaccination_model_import import VaccinationImport, VaccinationFlat
from flask_covid19.blueprints.app_web.web_model_transient import WebPageContent


app_vaccination = Blueprint('vaccination', __name__, template_folder='templates', url_prefix='/vaccination')

admin.add_view(ModelView(VaccinationImport, db.session, category="Vaccination"))
admin.add_view(ModelView(VaccinationFlat, db.session, category="Vaccination"))
admin.add_view(ModelView(VaccinationDateReported, db.session, category="Vaccination"))
admin.add_view(ModelView(VaccinationData, db.session, category="Vaccination"))


# ---------------------------------------------------------------------------------------------------------------
#  Url Routes Frontend
# ---------------------------------------------------------------------------------------------------------------

@app_vaccination.route('/info')
def url_vaccination_info():
    page_info = WebPageContent('Vaccination', "Info")
    return render_template(
        'rki_vaccination/rki_vaccination_info.html',
        page_info=page_info)


@app_vaccination.route('/tasks')
def url_vaccination_tasks():
    page_info = WebPageContent('Vaccination', "Tasks")
    return render_template(
        'rki_vaccination/rki_vaccination_tasks.html',
        page_info=page_info)


@app_vaccination.route('/imported/page/<int:page>')
@app_vaccination.route('/imported')
def url_vaccination_imported(page=1):
    page_info = WebPageContent('Vaccination', "Data: Germany Timeline imported")
    page_data = VaccinationImport.get_all_as_page(page)
    return render_template(
        'rki_vaccination/imported/rki_vaccination_imported.html',
        page_data=page_data,
        page_info=page_info)


@app_vaccination.route('/data/page/<int:page>')
@app_vaccination.route('/data')
def url_vaccination_data(page=1):
    page_info = WebPageContent('Vaccination', "Data: Germany Timeline")
    page_data = VaccinationData.get_all_as_page(page)
    return render_template(
        'rki_vaccination/data/rki_vaccination_data.html',
        page_data=page_data,
        page_info=page_info)


# ----------------------------------------------------------------------------------------------------------------
#  Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@celery.task(bind=True)
def task_vaccination_import_file(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_vaccination_import [OK] ")
    logger.info("------------------------------------------------------------")
    vaccination_service.import_file()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_vaccination_import)"
    return result


@celery.task(bind=True)
def task_vaccination_full_update_dimension_tables(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_vaccination_full_update_dimension_tables [OK] ")
    logger.info("------------------------------------------------------------")
    vaccination_service.full_update_dimension_tables()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_vaccination_full_update_dimension_tables)"
    return result


@celery.task(bind=True)
def task_vaccination_update_dimension_tables(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_vaccination_update_dimension_tables [OK] ")
    logger.info("------------------------------------------------------------")
    vaccination_service.update_dimension_tables()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_vaccination_update_dimension_tables)"
    return result


@celery.task(bind=True)
def task_vaccination_update_facttable(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_vaccination_update_facttable [OK] ")
    logger.info("------------------------------------------------------------")
    vaccination_service.update_fact_table()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_vaccination_update_facttable)"
    return result


@celery.task(bind=True)
def task_vaccination_full_update_facttable(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_vaccination_full_update_facttable [OK] ")
    logger.info("------------------------------------------------------------")
    vaccination_service.full_update_fact_table()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_vaccination_full_update_facttable)"
    return result


@celery.task(bind=True)
def task_vaccination_update_starschema(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_vaccination_update_starschema [OK] ")
    logger.info("------------------------------------------------------------")
    vaccination_service.full_update_star_schema()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_vaccination_update_starschema)"
    return result


@celery.task(bind=True)
def task_vaccination_full_update_starschema(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_vaccination_full_update_starschema [OK] ")
    logger.info("------------------------------------------------------------")
    vaccination_service.full_update_star_schema()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_vaccination_full_update_starschema)"
    return result


@celery.task(bind=True)
def task_vaccination_update(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_vaccination_update [OK] ")
    logger.info("------------------------------------------------------------")
    vaccination_service.full_update()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_vaccination_update)"
    return result


@celery.task(bind=True)
def task_vaccination_full_update(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_vaccination_full_update [OK] ")
    logger.info("------------------------------------------------------------")
    vaccination_service.full_update()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_vaccination_full_update)"
    return result

# ----------------------------------------------------------------------------------------------------------------
# URL Routes for Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@app_vaccination.route('/task/download')
def url_task_vaccination_download():
    flash("vaccination_service.download started")
    vaccination_service.download()
    flash("vaccination_service.download done")
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_vaccination.route('/task/import')
def url_task_vaccination_import():
    task_vaccination_import_file.apply_async()
    flash("task_vaccination_import_file started")
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_vaccination.route('/task/update/full/dimension-tables')
def url_task_vaccination_full_update_dimension_tables():
    flash("url_vaccination_task_update_dimensiontables_only started")
    task_vaccination_full_update_dimension_tables.apply_async()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_vaccination.route('/task/update/dimension-tables')
def url_task_vaccination_update_dimension_tables():
    flash("url_vaccination_task_update_dimensiontables_only started")
    task_vaccination_update_dimension_tables.apply_async()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_vaccination.route('/task/update/fact-table/incremental/only')
def url_task_vaccination_update_facttable():
    flash("url_vaccination_task_update_facttable_incremental_only started")
    task_vaccination_update_facttable.apply_async()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_vaccination.route('/task/update/fact-table/initial/only')
def url_task_vaccination_full_update_facttable():
    flash("url_vaccination_task_update_facttable_initial_only started")
    task_vaccination_full_update_facttable.apply_async()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_vaccination.route('/task/full/update/star_schema')
def url_task_vaccination_full_update_starschema():
    flash("url_vaccination_task_update_star_schemainitial started")
    task_vaccination_full_update_starschema.apply_async()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_vaccination.route('/task/update/star_schema')
def url_task_vaccination_update_starschema():
    flash("url_vaccination_task_update_starschema_incremental started")
    task_vaccination_update_starschema.apply_async()
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_vaccination.route('/task/full/update/')
def url_task_vaccination_full_update():
    vaccination_service.download()
    flash("vaccination_service.download done")
    task_vaccination_full_update.apply_async()
    flash("task_vaccination_full_update started")
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))


@app_vaccination.route('/task/update')
def url_task_vaccination_update():
    vaccination_service.download()
    flash("vaccination_service.download done")
    task_vaccination_update.apply_async()
    flash("task_vaccination_update started")
    return redirect(url_for('rki_vaccination.url_vaccination_tasks'))
