from sqlalchemy import and_
from sqlalchemy.orm import joinedload
from database import db, ITEMS_PER_PAGE
from flask_covid19.blueprints.app_all.all_model import BlueprintDateReported, BlueprintFactTable
from flask_covid19.blueprints.app_all.all_model import BlueprintLocationGroup, BlueprintLocation


class WhoDateReported(BlueprintDateReported):
    __tablename__ = 'who_date_reported'
    __mapper_args__ = {'concrete': True}
    __table_args__ = (
        db.UniqueConstraint(
            'date_reported_import_str',
            'datum',
            'year_day_of_year',
            name="uix_who_date_reported"
        ),
    )

    def __str__(self):
        return self.datum.isoformat()

    id = db.Column(db.Integer, primary_key=True)
    datum = db.Column(db.Date, nullable=False, unique=True)
    year = db.Column(db.Integer, nullable=False)
    month = db.Column(db.Integer, nullable=False)
    day_of_month = db.Column(db.Integer, nullable=False)
    day_of_week = db.Column(db.Integer, nullable=False)
    week_of_year = db.Column(db.Integer, nullable=False)
    day_of_year = db.Column(db.Integer, nullable=True)
    year_week = db.Column(db.String(255), nullable=False)
    year_day_of_year = db.Column(db.String(255), nullable=False, unique=True)
    date_reported_import_str = db.Column(db.String(255), nullable=False, unique=True)
    processed_update = db.Column(db.Boolean, nullable=False)
    processed_full_update = db.Column(db.Boolean, nullable=False)

    @classmethod
    def create_new_object_factory(cls, my_date_rep: str):
        o = cls.create_new_object_factory_for_who(my_date_rep)
        oo = WhoDateReported(
            datum=o.datum,
            year=o.year,
            month=o.month,
            day_of_month=o.day_of_month,
            day_of_week=o.day_of_week,
            week_of_year=o.week_of_year,
            day_of_year=o.day_of_year,
            year_week=o.year_week,
            year_day_of_year=o.year_day_of_year,
            date_reported_import_str=o.date_reported_import_str,
            processed_update=o.processed_update,
            processed_full_update=o.processed_full_update,
        )
        return oo


class WhoCountryRegion(BlueprintLocationGroup):
    __tablename__ = 'who_country_region'
    __mapper_args__ = {'concrete': True}
    __table_args__ = (
        db.UniqueConstraint('location_group', name="uix_who_country_region"),
    )

    id = db.Column(db.Integer, primary_key=True)
    location_group = db.Column(db.String(255), nullable=False, unique=True)
    processed_update = db.Column(db.Boolean, nullable=False)
    processed_full_update = db.Column(db.Boolean, nullable=False)

    def __str__(self):
        return " " + self.location_group + " "


class WhoCountry(BlueprintLocation):
    __tablename__ = 'who_country'
    __mapper_args__ = {'concrete': True}
    __table_args__ = (
        db.UniqueConstraint('location_code', 'location', name="uix_who_country"),
    )

    id = db.Column(db.Integer, primary_key=True)
    location_code = db.Column(db.String(255), unique=True, nullable=False)
    location = db.Column(db.String(255), unique=True, nullable=False)
    location_group_id = db.Column(db.Integer, db.ForeignKey('who_country_region.id'), nullable=False)
    location_group = db.relationship(
        'WhoCountryRegion',
        lazy='joined',
        cascade='save-update',
        order_by='WhoCountryRegion.location_group')
    processed_update = db.Column(db.Boolean, nullable=False)
    processed_full_update = db.Column(db.Boolean, nullable=False)

    def __str__(self):
        return self.location_code + " | " + self.location + " -> " + self.location_group.__str__()

    @classmethod
    def find_by_country_code_and_country_and_who_region_id(cls, i_country_code, i_country, my_region):
        return db.session.query(cls).filter(
            and_(
                cls.location_code == i_country_code,
                cls.location == i_country,
                cls.region_id == my_region.id
            )
        ).one_or_none()

    @classmethod
    def get_by_country_code_and_country(cls, s_country_code: str, s_country: str):
        return db.session.query(cls).filter(
            and_(
                cls.location_code == s_country_code,
                cls.location == s_country
            )
        ).one()

    @classmethod
    def find_by_country_code(cls, i_country_code):
        return db.session.query(cls).filter(
            cls.location_code == i_country_code
        ).one_or_none()

    @classmethod
    # @cache.memoize(50)
    def get_by_country_code(cls, i_country_code):
        return db.session.query(cls).filter(
            cls.location_code == i_country_code
        ).one()

    @classmethod
    def find_by_country(cls, i_country):
        return db.session.query(cls).filter(
            cls.location == i_country
        ).one_or_none()

    @classmethod
    # @cache.memoize(50)
    def get_by_country(cls, i_country):
        return db.session.query(cls).filter(
            cls.location == i_country
        ).one()

    @classmethod
    def get_who_countries_for_region(cls, region, page):
        return db.session.query(cls).filter(
            cls.location_group == region
        ).order_by(cls.location).paginate(page, per_page=ITEMS_PER_PAGE)


class WhoData(BlueprintFactTable):
    __tablename__ = 'who'
    __mapper_args__ = {'concrete': True}

    id = db.Column(db.Integer, primary_key=True)
    cases_new = db.Column(db.Integer, nullable=False)
    cases_cumulative = db.Column(db.Integer, nullable=False)
    deaths_new = db.Column(db.Integer, nullable=False)
    deaths_cumulative = db.Column(db.Integer, nullable=False)
    date_reported_id = db.Column(db.Integer, db.ForeignKey('who_date_reported.id'), nullable=False)
    date_reported = db.relationship(
        'WhoDateReported',
        lazy='joined',
        cascade='save-update',
        order_by='desc(WhoDateReported.datum)')
    location_id = db.Column(db.Integer, db.ForeignKey('who_country.id'), nullable=False)
    location = db.relationship(
        'WhoCountry',
        lazy='joined',
        cascade='save-update',
        order_by='asc(WhoCountry.location)')
    processed_update = db.Column(db.Boolean, nullable=False)
    processed_full_update = db.Column(db.Boolean, nullable=False)

    def __str__(self):
        return self.date_reported.__str__() + " " + self.location.__str__()

    @classmethod
    def find_one_or_none_by_date_and_country(cls, my_date_reported, my_country):
        return db.session.query(cls).filter(
            and_(
                cls.date_reported_id == my_date_reported.id,
                cls.location_id == my_country.id
            )
        ).one_or_none()

    @classmethod
    def get_data_for_country(cls, who_country, page):
        return db.session.query(cls).filter(
            cls.location_id == who_country.id
        ).populate_existing().options(
            joinedload(cls.location).joinedload(WhoCountry.location_group),
            joinedload(cls.date_reported)
        ).paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_data_for_day(cls, date_reported, page):
        return db.session.query(cls).filter(
                cls.date_reported_id == date_reported.id
            ).populate_existing().options(
                joinedload(cls.location).joinedload(WhoCountry.location_group),
                joinedload(cls.date_reported)
            ).order_by(
                cls.deaths_new.desc(),
                cls.cases_new.desc(),
                cls.deaths_cumulative.desc(),
                cls.cases_cumulative.desc()
            ).paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_data_for_one_day(cls, date_reported):
        return db.session.query(cls).filter(
            cls.date_reported_id == date_reported.id
        ).populate_existing().options(
            joinedload(cls.location).joinedload(WhoCountry.location_group),
            joinedload(cls.date_reported)
        ).order_by(
            cls.deaths_new.desc(),
            cls.cases_new.desc(),
            cls.deaths_cumulative.desc(),
            cls.cases_cumulative.desc()
        ).all()

    @classmethod
    def delete_data_for_one_day(cls, date_reported):
        for one_who_date in cls.get_data_for_one_day(date_reported):
            db.session.delete(one_who_date)
            db.session.commit()
        return None

    @classmethod
    def get_data_for_day_order_by_cases_new(cls, date_reported, page):
        return db.session.query(cls).filter(
                cls.date_reported_id == date_reported.id
            ).populate_existing().options(
                joinedload(cls.location).joinedload(WhoCountry.location_group),
                joinedload(cls.date_reported)
            ).order_by(
                cls.cases_new.desc()
            ).paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_data_for_day_order_by_cases_cumulative(cls, date_reported, page):
        return db.session.query(cls).filter(
                cls.date_reported_id == date_reported.id
            ).populate_existing().options(
                joinedload(cls.location).joinedload(WhoCountry.location_group),
                joinedload(cls.date_reported)
            ).order_by(
                cls.cases_cumulative.desc()
            ).paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_data_for_day_order_by_deaths_new(cls, date_reported, page):
        return db.session.query(cls).filter(
                cls.date_reported_id == date_reported.id
            ).populate_existing().options(
                joinedload(cls.location).joinedload(WhoCountry.location_group),
                joinedload(cls.date_reported)
            ).order_by(
                cls.deaths_new.desc()
            ).paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_data_for_day_order_by_deaths_cumulative(cls, date_reported, page):
        return db.session.query(cls).filter(
                cls.date_reported_id == date_reported.id
            ).populate_existing().options(
                joinedload(cls.location).joinedload(WhoCountry.location_group),
                joinedload(cls.date_reported)
            ).order_by(
                cls.deaths_cumulative.desc()
            ).paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_data_for_country_order_by_cases_new(cls, who_country, page):
        return db.session.query(cls).filter(
            cls.country_id == who_country.id
        ).populate_existing().options(
            joinedload(cls.location).joinedload(WhoCountry.location_group),
            joinedload(cls.date_reported)
        ).order_by(
            cls.cases_new.desc()
        ).paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_data_for_country_order_by_cases_cumulative(cls, who_country, page):
        return db.session.query(cls).filter(
            cls.country_id == who_country.id
        ).populate_existing().options(
            joinedload(cls.location).joinedload(WhoCountry.location_group),
            joinedload(cls.date_reported)
        ).order_by(
            cls.cases_cumulative.desc()
        ).paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_data_for_country_order_by_deaths_new(cls, who_country, page):
        return db.session.query(cls).filter(
            cls.country_id == who_country.id
        ).populate_existing().options(
            joinedload(cls.location).joinedload(WhoCountry.location_group),
            joinedload(cls.date_reported)
        ).order_by(
            cls.deaths_new.desc()
        ).paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_data_for_country_order_by_deaths_cumulative(cls, who_country, page):
        return db.session.query(cls).filter(
            cls.country_id == who_country.id
        ).populate_existing().options(
            joinedload(cls.location).joinedload(WhoCountry.location_group),
            joinedload(cls.date_reported)
        ).order_by(
            cls.deaths_cumulative.desc()
        ).paginate(page, per_page=ITEMS_PER_PAGE)
